@@ -3,7 +3,7 @@ use strict;
 use warnings;
 use autodie;
 
-use 5.008001;
+use 5.010000;
 
 use Carp;
 use IO::Zlib;
@@ -15,18 +15,6 @@ use YAML::Syck;
 
 use AlignDB::IntSpan;
 
-use base 'Exporter';
-use vars qw(@ISA @EXPORT_OK %EXPORT_TAGS);
-@ISA = qw(Exporter);
-%EXPORT_TAGS = (
-    all => [
-        qw{
-            read_sizes read_names new_set runlist2set decode_header
-            },
-    ],
-);
-@EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
-
 # The only entrance for AlignDB::IntSpan or AlignDB::IntSpanXS
 sub new_set {
     return AlignDB::IntSpan->new;
@@ -75,47 +63,51 @@ sub runlist2set {
 sub decode_header {
     my $header = shift;
 
-    # S288C.chrI(+):27070-29557|species=S288C
+    tie my %info, "Tie::IxHash";
+
+    # S288.chrI(+):27070-29557|species=S288C
     my $head_qr = qr{
         (?:(?P<name>[\w_]+)\.)?
         (?P<chr_name>[\w-]+)
         (?:\((?P<chr_strand>.+)\))?
         [\:]                        # spacer
         (?P<chr_start>\d+)
-        [\_\-]                      # spacer
-        (?P<chr_end>\d+)
+        [\_\-]?                      # spacer
+        (?P<chr_end>\d+)?
     }xi;
 
-    tie my %info, "Tie::IxHash";
-
     $header =~ $head_qr;
-    my $name     = $1;
-    my $chr_name = $2;
+    my $chr_name  = $2;
+    my $chr_start = $4;
+    my $chr_end   = $5;
 
-    if ( defined $name or defined $chr_name ) {
+    if ( defined $chr_name and defined $chr_start ) {
+        if ( !defined $chr_end ) {
+            $chr_end = $chr_start;
+        }
         %info = (
-            name       => $name,
+            name       => $1,
             chr_name   => $chr_name,
             chr_strand => $3,
-            chr_start  => $4,
-            chr_end    => $5,
+            chr_start  => $chr_start,
+            chr_end    => $chr_end,
         );
-        if ( !defined $info{chr_strand} ) {
-            $info{chr_strand} = '+';
-        }
-        elsif ( $info{chr_strand} eq '1' ) {
-            $info{chr_strand} = '+';
-        }
-        elsif ( $info{chr_strand} eq '-1' ) {
-            $info{chr_strand} = '-';
+        if ( defined $info{chr_strand} ) {
+            if ( $info{chr_strand} eq '1' ) {
+                $info{chr_strand} = '+';
+            }
+            elsif ( $info{chr_strand} eq '-1' ) {
+                $info{chr_strand} = '-';
+            }
         }
     }
     else {
-        $name = $header;
+        $header =~ /^(\S+)/;
+        my $name = $1;
         %info = (
-            name       => undef,
-            chr_name   => 'chrUn',
-            chr_strand => '+',
+            name       => $name,
+            chr_name   => undef,
+            chr_strand => undef,
             chr_start  => undef,
             chr_end    => undef,
         );
@@ -135,4 +127,50 @@ sub decode_header {
     return \%info;
 }
 
+sub encode_header {
+    my $info           = shift;
+    my $only_essential = shift;
+
+    my $header;
+    if ( defined $info->{name} ) {
+        if ( defined $info->{chr_name} ) {
+            $header .= $info->{name};
+            $header .= "." . $info->{chr_name};
+        }
+        else {
+            $header .= $info->{name};
+        }
+    }
+    elsif ( defined $info->{chr_name} ) {
+        $header .= $info->{chr_name};
+    }
+
+    if ( defined $info->{chr_strand} ) {
+        $header .= "(" . $info->{chr_strand} . ")";
+    }
+    if ( defined $info->{chr_start} ) {
+        $header .= ":" . $info->{chr_start};
+        if ( $info->{chr_end} != $info->{chr_start} ) {
+            $header .= "-" . $info->{chr_end};
+        }
+    }
+
+    # additional keys
+    if ( !$only_essential ) {
+        my %essential = map { $_ => 1 } qw{name chr_name chr_strand chr_start chr_end seq full_seq};
+        my @parts;
+        for my $key ( sort keys %{$info} ) {
+            if ( !$essential{$key} ) {
+                push @parts, $key . "=" . $info->{$key};
+            }
+        }
+        if (@parts) {
+            my $additional = join ";", @parts;
+            $header .= "|" . $additional;
+        }
+    }
+
+    return $header;
+}
+
 1;
